"""Module providing reductions functions that can be applied to Fluent data
from one or across multiple remote Fluent sessions.

The following parameters are relevant for the reduction functions. The
expr parameter is not relevant to all reductions functions.

Parameters
----------
expr : Any
    Expression that can be either a string or an
    instance of a specific settings API named_expressions
    object. The expression can be a field variable or a
    a valid Fluent expression. A specified named expression
    can be handled for multiple solvers as long as the
    expression's definition is valid in each solver (it
    does not need to be created in each solver)
locations : Any
    A list of location strings, or an API object that can be
    resolved to a list of location strings
    (e.g., setup.boundary_conditions,
    or results.surfaces.plane_surface),
    or a list of such objects. If location strings are
    included in the list, then only string must be included
ctxt : Any, optional
    An optional API object (e.g., the root solver session
    object but any solver API object will suffice) to set
    the context of the call's execution. If the location
    objects are strings, then such a context is required
Returns
-------
float or List[float]
    The result of the reduction

Examples
--------

>>> from ansys.fluent.core.solver.function import reduction
>>> # Compute the area average of absolute pressure across all boundary
>>> # condition surfaces of the given solver
>>> reduction.area_average(
...     expr = "AbsolutePressure",
...     locations = solver.setup.boundary_conditions.velocity_inlet
... )
10623.0

>>> from ansys.fluent.core.solver.function import reduction
>>> # Compute the minimum of the square of velocity magnitude
>>> # for all pressure outlets across two solvers
>>> named_exprs = solver1.setup.named_expressions
>>> vsquared = named_exprs["vsquared"] = {}
>>> vsquared.definition = "VelocityMagnitude ** 2"
>>> reduction.minimum(
...     expr = vsquared,
...     locations = [
...         solver1.setup.boundary_conditions.pressure_outlet,
...         solver2.setup.boundary_conditions.pressure_outlet
...     ])
19.28151
"""

from numpy import array


class BadReductionRequest(Exception):
    def __init__(self, err):
        super().__init__(f"Could not complete reduction function request: {err}")


def _is_iterable(obj):
    return hasattr(type(obj), "__iter__")


def _expand_locn_container(locns):
    try:
        return [[locn, locns] for locn in locns]
    except TypeError as ex:
        raise BadReductionRequest(ex)


def _locn_name_and_obj(locn, locns):
    if isinstance(locn, str):
        return [locn, locns]
    # should call locn_get_name()
    if _is_iterable(locn):
        return _locn_names_and_objs(locn)
    else:
        return [locn.obj_name, locn]


def _locn_names_and_objs(locns):
    if _is_iterable(locns):
        names_and_objs = []
        for locn in locns:
            name_and_obj = _locn_name_and_obj(locn, locns)
            if _is_iterable(name_and_obj):
                if isinstance(name_and_obj[0], str):
                    names_and_objs.append(name_and_obj)
                else:
                    names_and_objs.extend(name_and_obj)
        return names_and_objs
    else:
        return _expand_locn_container(locns)


def _root(obj):
    return (
        None
        if isinstance(obj, list)
        else obj
        if not getattr(obj, "obj_name", None)
        else _root(obj._parent)
    )


def _validate_locn_list(locn_list, ctxt):
    if not all(locn[0] for locn in locn_list) and (
        any(locn[0] for locn in locn_list) or not ctxt
    ):
        raise BadReductionRequest("Invalid combination of arguments")


def _locns(locns, ctxt):
    locn_names_and_objs = _locn_names_and_objs(locns)
    locn_list = []
    for name, obj in locn_names_and_objs:
        root = _root(obj)
        found = False
        for locn in locn_list:
            if locn[0] is root:
                locn[1].append(name)
                found = True
                break
        if not found:
            locn_list.append([root, [name]])
    _validate_locn_list(locn_list, ctxt)
    return locn_list


def _eval_expr(solver, expr_str):
    named_exprs = solver.setup.named_expressions
    expr_name = "temp_expr_1"
    named_exprs[expr_name] = {}
    # request feature: anonymous name object creation
    expr_obj = named_exprs["temp_expr_1"]
    expr_obj.definition = expr_str
    val = expr_obj.get_value()
    named_exprs.pop(expr_name)
    return val


def _expr_to_expr_str(expr):
    return getattr(expr, "definition", expr) if expr is not None else expr


def _eval_reduction(solver, reduction, locations, expr=None):
    expr_str = _expr_to_expr_str(expr)
    return _eval_expr(
        solver,
        (
            f"{reduction}({locations})"
            if expr_str is None
            else f"{reduction}({expr_str},{locations})"
        ),
    )


def _extent_expression(f_string, extent_name, expr, locations, ctxt):
    locns = _locns(locations, ctxt)
    numerator = 0.0
    denominator = 0.0
    for solver, names in locns:
        solver = solver or _root(ctxt)
        val = _eval_reduction(solver, f_string, names, expr)
        extent = _eval_reduction(solver, extent_name, names) if len(locns) > 1 else 1
        try:
            numerator += val * extent
            denominator += extent
        except TypeError:
            raise RuntimeError(val)
    if denominator == 0.0:
        raise BadReductionRequest("Zero extent computed for average")
    return numerator / denominator


def _extent_moment_vector(f_string, expr, locations, ctxt):
    locns = _locns(locations, ctxt)
    total = array([0.0, 0.0, 0.0])
    for solver, names in locns:
        solver = solver or _root(ctxt)
        extent = _eval_reduction(solver, f_string, names, expr)
        try:
            total += array(extent)
        except TypeError:
            raise RuntimeError(extent)
    return total


def _extent_average(extent_name, expr, locations, ctxt):
    return _extent_expression(f"{extent_name}Ave", extent_name, expr, locations, ctxt)


def _extent_integrated_average(extent_name, expr, locations, ctxt):
    return _extent_expression(f"{extent_name}Int", extent_name, expr, locations, ctxt)


def _extent(extent_name, locations, ctxt):
    locns = _locns(locations, ctxt)
    total = 0.0
    for solver, names in locns:
        solver = solver or _root(ctxt)
        extent = _eval_expr(solver, f"{extent_name}({names})")
        try:
            total += extent
        except TypeError:
            raise RuntimeError(extent)
    return total


def _extent_vectors(extent_name, locations, ctxt):
    locns = _locns(locations, ctxt)
    total = array([0.0, 0.0, 0.0])
    for solver, names in locns:
        solver = solver or _root(ctxt)
        extent = _eval_expr(solver, f"{extent_name}({names})")
        try:
            total += array(extent)
        except TypeError:
            raise RuntimeError(extent)
    return total


def _limit(limit, expr, locations, ctxt):
    locns = _locns(locations, ctxt)
    limit_val = None
    for solver, names in locns:
        solver = solver or _root(ctxt)
        val = _eval_reduction(
            solver, "Minimum" if limit is min else "Maximum", names, expr
        )
        limit_val = val if limit_val is None else limit(val, limit_val)
    return limit_val


def area_average(expr, locations, ctxt=None):
    """Compute the area averaged value of the specified expression over the
    specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_average("Area", expr, locations, ctxt)


def area_integrated_average(expr, locations, ctxt=None):
    """Compute the area integrated averaged of the specified expression over
    the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_integrated_average("Area", expr, locations, ctxt)


def volume_average(expr, locations, ctxt=None):
    """Compute the volume-weighted average value of the specified expression
    over the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_average("Volume", expr, locations, ctxt)


def volume_integrated_average(expr, locations, ctxt=None):
    """Compute the volume-weighted total of the specified expression over the
    specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_integrated_average("Volume", expr, locations, ctxt)


def area(locations, ctxt=None):
    """Compute the total area of the specified locations.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent("Area", locations, ctxt)


def volume(locations, ctxt=None):
    """Compute the total volume of the specified locations.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent("Volume", locations, ctxt)


def count(locations, ctxt=None):
    """Compute the total number of cells included in the specified locations.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent("Count", locations, ctxt)


def centroid(locations, ctxt=None):
    """Compute the geometric centroid of the specified location(s) as a vector.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_vectors("Centroid", locations, ctxt)


def force(locations, ctxt=None):
    """Compute the force acting on the location(s) specified (should be walls)
    as a vector.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_vectors("Force", locations, ctxt)


def pressure_force(locations, ctxt=None):
    """Compute the pressure force acting on the location(s) specified (should
    be walls) as a vector.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_vectors("PressureForce", locations, ctxt)


def viscous_force(locations, ctxt=None):
    """Compute the viscous force acting on the location(s) specified (should be
    walls) as a vector.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_vectors("ViscousForce", locations, ctxt)


def moment(expr, locations, ctxt=None):
    """Compute  the moment vector about the specified point (which can be
    single-valued expression) for the specified location(s).

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_moment_vector("Moment", expr, locations, ctxt)


def minimum(expr, locations, ctxt=None):
    """Compute the minimum of the specified expression over the specified
    locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _limit(min, expr, locations, ctxt)


def maximum(expr, locations, ctxt=None):
    """Compute the maximum of the specified expression over the specified
    locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _limit(max, expr, locations, ctxt)


def mass_average(expr, locations, ctxt=None):
    """Compute the mass-weighted average value of the specified expression over
    the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_average("Mass", expr, locations, ctxt)


def mass_integrated_average(expr, locations, ctxt=None):
    """Compute the total mass-weighted value of the specified expression over
    the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_integrated_average("Mass", expr, locations, ctxt)


def mass_flow_average(expr, locations, ctxt=None):
    """Compute the mass-flow-weighted average value of the specified expression
    over the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_average("MassFlow", expr, locations, ctxt)


def mass_flow_integrated_average(expr, locations, ctxt=None):
    """Compute the total mass flow over the specified locations.

    Parameters
    ----------
    expr : Any
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent_integrated_average("MassFlow", expr, locations, ctxt)


def mass_flow(locations, ctxt=None):
    """Compute the total mass flow rate of the specified locations.

    Parameters
    ----------
    locations : Any
    ctxt : Any, optional
    Returns
    -------
    float
    """
    return _extent("MassFlow", locations, ctxt)



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .reactions_1 import reactions
from .reaction_source_term_relaxation_factor import reaction_source_term_relaxation_factor
from .numerics_pbns import numerics_pbns
from .numerics_dbns import numerics_dbns
class expert(Group):
    """
    Enter expert menu.
    """

    fluent_name = "expert"

    child_names = \
        ['reactions', 'reaction_source_term_relaxation_factor',
         'numerics_pbns', 'numerics_dbns']

    reactions: reactions = reactions
    """
    reactions child of expert.
    """
    reaction_source_term_relaxation_factor: reaction_source_term_relaxation_factor = reaction_source_term_relaxation_factor
    """
    reaction_source_term_relaxation_factor child of expert.
    """
    numerics_pbns: numerics_pbns = numerics_pbns
    """
    numerics_pbns child of expert.
    """
    numerics_dbns: numerics_dbns = numerics_dbns
    """
    numerics_dbns child of expert.
    """



"""Module containing class encapsulating Fluent connection."""

import importlib

from ansys.fluent.core.services.datamodel_se import PyMenuGeneric
from ansys.fluent.core.services.datamodel_tui import TUIMenuGeneric
from ansys.fluent.core.session import (
    _CODEGEN_MSG_TUI,
    _BaseSession,
    _get_preferences,
    _get_solverworkflow,
)
from ansys.fluent.core.session_shared import _CODEGEN_MSG_DATAMODEL
from ansys.fluent.core.solver.flobject import get_root as settings_get_root
from ansys.fluent.core.utils.fluent_version import get_version_for_filepath
from ansys.fluent.core.utils.logging import LOG
from ansys.fluent.core.workflow import WorkflowWrapper


class Solver(_BaseSession):
    """Encapsulates a Fluent - Solver session connection.
    Solver(Session) holds the top-level objects
    for solver TUI and settings objects calls."""

    def __init__(
        self,
        fluent_connection,
    ):
        super(Solver, self).__init__(fluent_connection=fluent_connection)
        self._tui_service = self.fluent_connection.datamodel_service_tui
        self._se_service = fluent_connection.datamodel_service_se
        self._settings_service = self.fluent_connection.settings_service
        self._tui = None
        self._workflow = None
        self._settings_root = None
        self._version = None
        self._solverworkflow = None

    @property
    def version(self):
        if self._version is None:
            self._version = get_version_for_filepath(session=self)
        return self._version

    @property
    def tui(self):
        """Instance of ``main_menu`` on which Fluent's SolverTUI methods can be
        executed."""
        if self._tui is None:
            try:
                tui_module = importlib.import_module(
                    f"ansys.fluent.core.solver.tui_{self.version}"
                )
                self._tui = tui_module.main_menu([], self._tui_service)
            except ImportError:
                LOG.warning(_CODEGEN_MSG_TUI)
                self._tui = TUIMenuGeneric([], self._tui_service)
        return self._tui

    @property
    def _workflow_se(self):
        """workflow datamodel root."""
        try:
            workflow_module = importlib.import_module(
                f"ansys.fluent.core.datamodel_{self.version}.workflow"
            )
            workflow_se = workflow_module.Root(self._se_service, "workflow", [])
        except (ImportError, ModuleNotFoundError):
            LOG.warning(_CODEGEN_MSG_DATAMODEL)
            workflow_se = PyMenuGeneric(self._se_service, "workflow")
        return workflow_se

    @property
    def workflow(self):
        if not self._workflow:
            self._workflow = WorkflowWrapper(self._workflow_se, Solver)
        return self._workflow

    @property
    def _root(self):
        """root settings object."""
        if self._settings_root is None:
            self._settings_root = settings_get_root(
                flproxy=self._settings_service, version=self.version
            )
        return self._settings_root

    @property
    def file(self):
        """file settings."""
        return self._root.file

    @property
    def mesh(self):
        """mesh settings."""
        return self._root.mesh

    @property
    def setup(self):
        """setup settings."""
        return self._root.setup

    @property
    def solution(self):
        """solution settings."""
        return self._root.solution

    @property
    def results(self):
        """results settings."""
        return self._root.results

    @property
    def parametric_studies(self):
        """parametric_studies settings."""
        return self._root.parametric_studies

    @property
    def current_parametric_study(self):
        """current_parametric_study settings."""
        return self._root.current_parametric_study

    @property
    def parallel(self):
        """parallel settings."""
        return self._root.parallel

    @property
    def report(self):
        """report settings."""
        return self._root.report

    @property
    def preferences(self):
        """preferences datamodel root."""
        if self._preferences is None:
            self._preferences = _get_preferences(self)
        return self._preferences

    @property
    def solverworkflow(self):
        """solverworkflow datamodel root."""
        if self._solverworkflow is None:
            self._solverworkflow = _get_solverworkflow(self)
        return self._solverworkflow



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .expert import expert
from .relative_convergence_criterion import relative_convergence_criterion
from .max_iter_per_timestep_count import max_iter_per_timestep_count
class acoustics_wave_eqn_controls(Group):
    """
    Enter menu for acoustics wave equation solver controls.
    """

    fluent_name = "acoustics-wave-eqn-controls"

    child_names = \
        ['expert', 'relative_convergence_criterion',
         'max_iter_per_timestep_count']

    expert: expert = expert
    """
    expert child of acoustics_wave_eqn_controls.
    """
    relative_convergence_criterion: relative_convergence_criterion = relative_convergence_criterion
    """
    relative_convergence_criterion child of acoustics_wave_eqn_controls.
    """
    max_iter_per_timestep_count: max_iter_per_timestep_count = max_iter_per_timestep_count
    """
    max_iter_per_timestep_count child of acoustics_wave_eqn_controls.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .material import material
from .sources import sources
from .source_terms_1 import source_terms
from .fixed import fixed
from .cylindrical_fixed_var import cylindrical_fixed_var
from .fixes import fixes
from .motion_spec import motion_spec
from .relative_to_thread import relative_to_thread
from .omega import omega
from .axis_origin_1 import axis_origin
from .axis_direction_1 import axis_direction
from .udf_zmotion_name import udf_zmotion_name
from .mrf_motion import mrf_motion
from .mrf_relative_to_thread import mrf_relative_to_thread
from .mrf_omega import mrf_omega
from .reference_frame_velocity import reference_frame_velocity
from .reference_frame_axis_origin import reference_frame_axis_origin
from .reference_frame_axis_direction import reference_frame_axis_direction
from .mrf_udf_zmotion_name import mrf_udf_zmotion_name
from .mgrid_enable_transient import mgrid_enable_transient
from .mgrid_motion import mgrid_motion
from .mgrid_relative_to_thread import mgrid_relative_to_thread
from .mgrid_omega import mgrid_omega
from .moving_mesh_velocity import moving_mesh_velocity
from .moving_mesh_axis_origin import moving_mesh_axis_origin
from .moving_mesh_axis_direction import moving_mesh_axis_direction
from .mgrid_udf_zmotion_name import mgrid_udf_zmotion_name
from .solid_motion import solid_motion
from .solid_relative_to_thread import solid_relative_to_thread
from .solid_omega import solid_omega
from .solid_motion_velocity import solid_motion_velocity
from .solid_motion_axis_origin import solid_motion_axis_origin
from .solid_motion_axis_direction import solid_motion_axis_direction
from .solid_udf_zmotion_name import solid_udf_zmotion_name
from .radiating import radiating
from .les_embedded import les_embedded
from .contact_property import contact_property
from .active_wetsteam_zone import active_wetsteam_zone
from .vapor_phase_realgas import vapor_phase_realgas
from .cursys import cursys
from .cursys_name import cursys_name
from .pcb_model import pcb_model
from .pcb_zone_info import pcb_zone_info
class phase_child(Group):
    """
    'child_object_type' of phase.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['material', 'sources', 'source_terms', 'fixed',
         'cylindrical_fixed_var', 'fixes', 'motion_spec',
         'relative_to_thread', 'omega', 'axis_origin', 'axis_direction',
         'udf_zmotion_name', 'mrf_motion', 'mrf_relative_to_thread',
         'mrf_omega', 'reference_frame_velocity',
         'reference_frame_axis_origin', 'reference_frame_axis_direction',
         'mrf_udf_zmotion_name', 'mgrid_enable_transient', 'mgrid_motion',
         'mgrid_relative_to_thread', 'mgrid_omega', 'moving_mesh_velocity',
         'moving_mesh_axis_origin', 'moving_mesh_axis_direction',
         'mgrid_udf_zmotion_name', 'solid_motion', 'solid_relative_to_thread',
         'solid_omega', 'solid_motion_velocity', 'solid_motion_axis_origin',
         'solid_motion_axis_direction', 'solid_udf_zmotion_name', 'radiating',
         'les_embedded', 'contact_property', 'active_wetsteam_zone',
         'vapor_phase_realgas', 'cursys', 'cursys_name', 'pcb_model',
         'pcb_zone_info']

    material: material = material
    """
    material child of phase_child.
    """
    sources: sources = sources
    """
    sources child of phase_child.
    """
    source_terms: source_terms = source_terms
    """
    source_terms child of phase_child.
    """
    fixed: fixed = fixed
    """
    fixed child of phase_child.
    """
    cylindrical_fixed_var: cylindrical_fixed_var = cylindrical_fixed_var
    """
    cylindrical_fixed_var child of phase_child.
    """
    fixes: fixes = fixes
    """
    fixes child of phase_child.
    """
    motion_spec: motion_spec = motion_spec
    """
    motion_spec child of phase_child.
    """
    relative_to_thread: relative_to_thread = relative_to_thread
    """
    relative_to_thread child of phase_child.
    """
    omega: omega = omega
    """
    omega child of phase_child.
    """
    axis_origin: axis_origin = axis_origin
    """
    axis_origin child of phase_child.
    """
    axis_direction: axis_direction = axis_direction
    """
    axis_direction child of phase_child.
    """
    udf_zmotion_name: udf_zmotion_name = udf_zmotion_name
    """
    udf_zmotion_name child of phase_child.
    """
    mrf_motion: mrf_motion = mrf_motion
    """
    mrf_motion child of phase_child.
    """
    mrf_relative_to_thread: mrf_relative_to_thread = mrf_relative_to_thread
    """
    mrf_relative_to_thread child of phase_child.
    """
    mrf_omega: mrf_omega = mrf_omega
    """
    mrf_omega child of phase_child.
    """
    reference_frame_velocity: reference_frame_velocity = reference_frame_velocity
    """
    reference_frame_velocity child of phase_child.
    """
    reference_frame_axis_origin: reference_frame_axis_origin = reference_frame_axis_origin
    """
    reference_frame_axis_origin child of phase_child.
    """
    reference_frame_axis_direction: reference_frame_axis_direction = reference_frame_axis_direction
    """
    reference_frame_axis_direction child of phase_child.
    """
    mrf_udf_zmotion_name: mrf_udf_zmotion_name = mrf_udf_zmotion_name
    """
    mrf_udf_zmotion_name child of phase_child.
    """
    mgrid_enable_transient: mgrid_enable_transient = mgrid_enable_transient
    """
    mgrid_enable_transient child of phase_child.
    """
    mgrid_motion: mgrid_motion = mgrid_motion
    """
    mgrid_motion child of phase_child.
    """
    mgrid_relative_to_thread: mgrid_relative_to_thread = mgrid_relative_to_thread
    """
    mgrid_relative_to_thread child of phase_child.
    """
    mgrid_omega: mgrid_omega = mgrid_omega
    """
    mgrid_omega child of phase_child.
    """
    moving_mesh_velocity: moving_mesh_velocity = moving_mesh_velocity
    """
    moving_mesh_velocity child of phase_child.
    """
    moving_mesh_axis_origin: moving_mesh_axis_origin = moving_mesh_axis_origin
    """
    moving_mesh_axis_origin child of phase_child.
    """
    moving_mesh_axis_direction: moving_mesh_axis_direction = moving_mesh_axis_direction
    """
    moving_mesh_axis_direction child of phase_child.
    """
    mgrid_udf_zmotion_name: mgrid_udf_zmotion_name = mgrid_udf_zmotion_name
    """
    mgrid_udf_zmotion_name child of phase_child.
    """
    solid_motion: solid_motion = solid_motion
    """
    solid_motion child of phase_child.
    """
    solid_relative_to_thread: solid_relative_to_thread = solid_relative_to_thread
    """
    solid_relative_to_thread child of phase_child.
    """
    solid_omega: solid_omega = solid_omega
    """
    solid_omega child of phase_child.
    """
    solid_motion_velocity: solid_motion_velocity = solid_motion_velocity
    """
    solid_motion_velocity child of phase_child.
    """
    solid_motion_axis_origin: solid_motion_axis_origin = solid_motion_axis_origin
    """
    solid_motion_axis_origin child of phase_child.
    """
    solid_motion_axis_direction: solid_motion_axis_direction = solid_motion_axis_direction
    """
    solid_motion_axis_direction child of phase_child.
    """
    solid_udf_zmotion_name: solid_udf_zmotion_name = solid_udf_zmotion_name
    """
    solid_udf_zmotion_name child of phase_child.
    """
    radiating: radiating = radiating
    """
    radiating child of phase_child.
    """
    les_embedded: les_embedded = les_embedded
    """
    les_embedded child of phase_child.
    """
    contact_property: contact_property = contact_property
    """
    contact_property child of phase_child.
    """
    active_wetsteam_zone: active_wetsteam_zone = active_wetsteam_zone
    """
    active_wetsteam_zone child of phase_child.
    """
    vapor_phase_realgas: vapor_phase_realgas = vapor_phase_realgas
    """
    vapor_phase_realgas child of phase_child.
    """
    cursys: cursys = cursys
    """
    cursys child of phase_child.
    """
    cursys_name: cursys_name = cursys_name
    """
    cursys_name child of phase_child.
    """
    pcb_model: pcb_model = pcb_model
    """
    pcb_model child of phase_child.
    """
    pcb_zone_info: pcb_zone_info = pcb_zone_info
    """
    pcb_zone_info child of phase_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .select_sample import select_sample
from .plotting_grid_interval_size import plotting_grid_interval_size
from .prepare_expressions import prepare_expressions
class dpm_sample_contour_plots(Group):
    """
    'dpm_sample_contour_plots' child.
    """

    fluent_name = "dpm-sample-contour-plots"

    child_names = \
        ['select_sample', 'plotting_grid_interval_size']

    select_sample: select_sample = select_sample
    """
    select_sample child of dpm_sample_contour_plots.
    """
    plotting_grid_interval_size: plotting_grid_interval_size = plotting_grid_interval_size
    """
    plotting_grid_interval_size child of dpm_sample_contour_plots.
    """
    command_names = \
        ['prepare_expressions']

    prepare_expressions: prepare_expressions = prepare_expressions
    """
    prepare_expressions command of dpm_sample_contour_plots.
    """



"""Reader for Fluent case files.

Example
-------

from ansys.fluent.core.filereader.casereader import CaseReader

Instantiate a case reader

reader = CaseReader(case_filepath=case_filepath)

Get lists of input and output parameters

input_parameters = reader.input_parameters()
output_parameters = reader.output_parameters()
"""
import codecs
import glob
import gzip
import itertools
from os.path import dirname
from pathlib import Path
from typing import List

import h5py

from . import lispy


class InputParameter:
    """Class to represent an input parameter.

    Attributes
    ----------
    name : str
    value
        The value of this input parameter, usually
        a string, qualified by units
    """

    def __init__(self, raw_data):
        self.name, self.name = None, None
        for k, v in raw_data:
            if k == "name":
                self.name = v
            elif k == "definition":
                self.value = v


class OutputParameter:
    """Class to represent an output parameter.

    Attributes
    ----------
    name : str
    """

    def __init__(self, raw_data):
        parameter = raw_data[1]
        for elem in parameter:
            if len(elem) and elem[0] == "name":
                self.name = elem[1][1]
            if len(elem) and elem[0] == "fluent-units":
                self.units = elem[1].strip()


class _CaseVariable:
    def __init__(self, variables: dict, path: str = ""):
        self._variables = variables
        self._path = path

    def __call__(self, name: str = ""):
        if not name:
            error_name = self._path[:-1] if self._path else self._path
            raise RuntimeError(f"Invalid variable {error_name}")
        return self._variables[name]

    def __getattr__(self, name: str):
        for orig, sub in (
            ("__q", "?"),
            ("__dot", "."),
            ("__plus", "+"),
            ("_", "-"),
        ):
            name = name.replace(orig, sub)
        try:
            name = self._path + name
            result = self._variables[name]
            return lambda: result
        except KeyError:
            return _CaseVariable(self._variables, name + "/")


class CaseReader:
    """Class to read a Fluent case file.

    Methods
    -------
    input_parameters
        Get a list of input parameter objects
    output_parameters
        Get a list of output parameter objects
    num_dimensions
        Get the dimensionality of the case (2 or 3)
    precision
        Get the precision (1 or 2 for 1D of 2D)
    rp_vars
        Get dictionary of all RP vars
    rp_var
        Get specific RP var by name, either by providing
        the Scheme name:
            `reader.rp_var("rad/enable-netm?")`
        or a pythonic version:
            `reader.rp_var.rad.enable_netm__q()`
    has_rp_var
        Whether case has particular RP var
    config_vars
        Get dictionary of all RP vars
    config_var
        Get specific config var by name, either by providing
        the Scheme name:
            `reader.config_var("rp-3d?")`
        or a pythonic version:
            `reader.config_var.rp_3d__q()`
    has_config_var
        Whether case has particular config var
    """

    def __init__(self, case_filepath: str = None, project_filepath: str = None):
        if case_filepath and project_filepath:
            raise RuntimeError(
                "Please enter either the case file path or the project file path"
            )
        if project_filepath:
            if Path(project_filepath).suffix in [".flprj", ".flprz"]:
                case_filepath = _get_case_filepath(dirname(project_filepath))
            else:
                raise RuntimeError("Please provide a valid fluent project file path")
        try:
            if "".join(Path(case_filepath).suffixes) == ".cas.h5":
                file = h5py.File(case_filepath)
                settings = file["settings"]
                rpvars = settings["Rampant Variables"][0]
                rp_vars_str = rpvars.decode()
            elif Path(case_filepath).suffix == ".cas":
                with open(case_filepath, "rb") as file:
                    rp_vars_str = file.read()
                rp_vars_str = _get_processed_string(rp_vars_str)
            elif "".join(Path(case_filepath).suffixes) == ".cas.gz":
                with gzip.open(case_filepath, "rb") as file:
                    rp_vars_str = file.read()
                rp_vars_str = _get_processed_string(rp_vars_str)
            else:
                raise RuntimeError()

        except FileNotFoundError:
            raise RuntimeError(f"The case file {case_filepath} cannot be found.")

        except OSError:
            error_message = (
                "Could not read case file. "
                "Only valid Case files (.h5, .cas, .cas.gz) can be read. "
            )
            raise RuntimeError(error_message)

        except BaseException:
            raise RuntimeError(f"Could not read case file {case_filepath}")

        self._rp_vars = {v[0]: v[1] for v in lispy.parse(rp_vars_str)[1]}

        self._config_vars = {v[0]: v[1] for v in self._rp_vars["case-config"]}

    def input_parameters(self) -> List[InputParameter]:
        exprs = self._named_expressions()
        if exprs:
            input_params = []
            for expr in exprs:
                for attr in expr:
                    if attr[0] in ["parameter", "input-parameter"] and attr[1] is True:
                        input_params.append(InputParameter(expr))
            return input_params
        else:
            parameters = self._find_rp_var("parameters/input-parameters")
            return [InputParameter(param) for param in parameters]

    def output_parameters(self) -> List[OutputParameter]:
        parameters = self._find_rp_var("parameters/output-parameters")
        return [OutputParameter(param) for param in parameters]

    def num_dimensions(self) -> int:
        for attr in self._case_config():
            if attr[0] == "rp-3d?":
                return 3 if attr[1] is True else 2

    def precision(self) -> int:
        for attr in self._case_config():
            if attr[0] == "rp-double?":
                return 2 if attr[1] is True else 1

    def rp_vars(self) -> dict:
        return self._rp_vars

    @property
    def rp_var(self):
        return _CaseVariable(self._rp_vars)

    def has_rp_var(self, name):
        return name in self._rp_vars

    def config_vars(self):
        return self._config_vars

    @property
    def config_var(self):
        return _CaseVariable(self._config_vars)

    def has_config_var(self, name):
        return name in self._config_vars

    def _named_expressions(self):
        return self._find_rp_var("named-expressions")

    def _case_config(self):
        return self._find_rp_var("case-config")

    def _find_rp_var(self, name: str):
        return self._rp_vars[name]


def _get_processed_string(input_string: bytes) -> str:
    """Processes the input string (binary) with help of an identifier to return
    it in a format which can be parsed by lispy.parse()

    Parameters
    ----------
    input_string : bytes
        The input string in bytes

    Returns
    -------
    processed string (str)
    """
    rp_vars_str = codecs.decode(input_string, errors="ignore")
    string_identifier = "(37 ("
    return string_identifier + rp_vars_str.split(string_identifier)[1]


def _get_case_filepath(project_dir_path: str) -> str:
    """Gets case file path within the provided project directory path.

    Parameters
    ----------
    project_dir_path : str
        The directory containing the case file

    Returns
    -------
    case file path (str)
    """
    file_list = list(
        itertools.chain(
            *(
                glob.glob(project_dir_path + r"/**/**-Solve/*.%s" % ext)
                for ext in ["cas", "cas.h5", "cas.gz"]
            )
        )
    )
    if len(file_list) < 1:
        raise RuntimeError(f"No case files are present in: {project_dir_path}")
    elif len(file_list) > 1:
        raise RuntimeError(f"More than one case file is present in: {project_dir_path}")
    else:
        return file_list[0]



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .axis_direction_component_child import axis_direction_component_child

class axis_direction_component(ListObject[axis_direction_component_child]):
    """
    'axis_direction_component' child.
    """

    fluent_name = "axis-direction-component"

    child_object_type: axis_direction_component_child = axis_direction_component_child
    """
    child_object_type of axis_direction_component.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .local_dt_dualts_relax import local_dt_dualts_relax
from .global_dt_pseudo_relax import global_dt_pseudo_relax
class pseudo_time_explicit_relaxation_factor(Group):
    """
    'pseudo_time_explicit_relaxation_factor' child.
    """

    fluent_name = "pseudo-time-explicit-relaxation-factor"

    child_names = \
        ['local_dt_dualts_relax', 'global_dt_pseudo_relax']

    local_dt_dualts_relax: local_dt_dualts_relax = local_dt_dualts_relax
    """
    local_dt_dualts_relax child of pseudo_time_explicit_relaxation_factor.
    """
    global_dt_pseudo_relax: global_dt_pseudo_relax = global_dt_pseudo_relax
    """
    global_dt_pseudo_relax child of pseudo_time_explicit_relaxation_factor.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .filename import filename
class define_macro(Command):
    """
    Save input to a named macro.
    
    Parameters
    ----------
        filename : str
            'filename' child.
    
    """

    fluent_name = "define-macro"

    argument_names = \
        ['filename']

    filename: filename = filename
    """
    filename argument of define_macro.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .averaging import averaging
from .source_terms import source_terms
from .tracking import tracking
class numerics(Group):
    """
    Main menu to allow users to set options controlling the solution of ordinary differential equations describing 
    the underlying physics of the Discrete Phase Model.
    For more details consult the help option of the corresponding menu or TUI command.
    """

    fluent_name = "numerics"

    child_names = \
        ['averaging', 'source_terms', 'tracking']

    averaging: averaging = averaging
    """
    averaging child of numerics.
    """
    source_terms: source_terms = source_terms
    """
    source_terms child of numerics.
    """
    tracking: tracking = tracking
    """
    tracking child of numerics.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .list import list
from .graphics_objects_child import graphics_objects_child

class graphics_objects(NamedObject[graphics_objects_child], _CreatableNamedObjectMixin[graphics_objects_child]):
    """
    'graphics_objects' child.
    """

    fluent_name = "graphics-objects"

    command_names = \
        ['list']

    list: list = list
    """
    list command of graphics_objects.
    """
    child_object_type: graphics_objects_child = graphics_objects_child
    """
    child_object_type of graphics_objects.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .cycle_type import cycle_type
from .termination_criteria import termination_criteria
from .residual_reduction_tolerance import residual_reduction_tolerance
from .method import method
from .stabilization import stabilization
class multi_grid_controls_child(Group):
    """
    'child_object_type' of multi_grid_controls.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['cycle_type', 'termination_criteria', 'residual_reduction_tolerance',
         'method', 'stabilization']

    cycle_type: cycle_type = cycle_type
    """
    cycle_type child of multi_grid_controls_child.
    """
    termination_criteria: termination_criteria = termination_criteria
    """
    termination_criteria child of multi_grid_controls_child.
    """
    residual_reduction_tolerance: residual_reduction_tolerance = residual_reduction_tolerance
    """
    residual_reduction_tolerance child of multi_grid_controls_child.
    """
    method: method = method
    """
    method child of multi_grid_controls_child.
    """
    stabilization: stabilization = stabilization
    """
    stabilization child of multi_grid_controls_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .scalar_parameters import scalar_parameters
from .coupled_parameters import coupled_parameters
from .flexible_cycle_parameters import flexible_cycle_parameters
from .options_2 import options
class amg_controls(Group):
    """
    'amg_controls' child.
    """

    fluent_name = "amg-controls"

    child_names = \
        ['scalar_parameters', 'coupled_parameters',
         'flexible_cycle_parameters', 'options']

    scalar_parameters: scalar_parameters = scalar_parameters
    """
    scalar_parameters child of amg_controls.
    """
    coupled_parameters: coupled_parameters = coupled_parameters
    """
    coupled_parameters child of amg_controls.
    """
    flexible_cycle_parameters: flexible_cycle_parameters = flexible_cycle_parameters
    """
    flexible_cycle_parameters child of amg_controls.
    """
    options: options = options
    """
    options child of amg_controls.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .two_stage_runge_kutta import two_stage_runge_kutta
from .default_multi_stage_runge_kutta import default_multi_stage_runge_kutta
class rk2(Group):
    """
    'rk2' child.
    """

    fluent_name = "rk2"

    child_names = \
        ['two_stage_runge_kutta', 'default_multi_stage_runge_kutta']

    two_stage_runge_kutta: two_stage_runge_kutta = two_stage_runge_kutta
    """
    two_stage_runge_kutta child of rk2.
    """
    default_multi_stage_runge_kutta: default_multi_stage_runge_kutta = default_multi_stage_runge_kutta
    """
    default_multi_stage_runge_kutta child of rk2.
    """



"""Module containing class encapsulating Fluent connection.

**********PRESENTLY SAME AS SOLVER WITH A SWITCH TO SOLVER***********
"""
from ansys.fluent.core.session_solver import Solver


class SolverLite(Solver):
    """Encapsulates a Fluent - Solver(Lite) session connection.
    SolverLite(Session) holds the top-level objects
    for solver TUI and settings objects calls."""

    def __init__(
        self,
        fluent_connection=None,
    ):
        super().__init__(
            fluent_connection=fluent_connection,
        )
        self._tui_service = self.fluent_connection.datamodel_service_tui
        self._settings_service = self.fluent_connection.settings_service
        self._tui = None
        self._settings_root = None

    # One can inherit methods from 'Solver' and re-define it here to make it unavailable in solver-Lite

    def switch_to_full_solver(self):
        """A switch to move to the full-solver session from solver-lite."""
        solver_session = Solver(fluent_connection=self.fluent_connection)
        return solver_session



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from .accelerated_non_iterative_time_marching import accelerated_non_iterative_time_marching
from .convergence_acceleration_for_stretched_meshes import convergence_acceleration_for_stretched_meshes
from .discretization_scheme import discretization_scheme
from .expert_2 import expert
from .flux_type_2 import flux_type
from .frozen_flux import frozen_flux
from .gradient_scheme import gradient_scheme
from .high_order_term_relaxation import high_order_term_relaxation
from .multiphase_numerics import multiphase_numerics
from .nb_gradient_boundary_option import nb_gradient_boundary_option
from .nita_expert_controls import nita_expert_controls
from .noniterative_time_advance import noniterative_time_advance
from .overset_1 import overset
from .p_v_coupling_1 import p_v_coupling
from .phase_based_vof_discretization import phase_based_vof_discretization
from .reduced_rank_extrapolation import reduced_rank_extrapolation
from .reduced_rank_extrapolation_options import reduced_rank_extrapolation_options
from .residual_smoothing import residual_smoothing
from .unsteady_1st_order import unsteady_1st_order
from .unsteady_2nd_order import unsteady_2nd_order
from .unsteady_2nd_order_bounded import unsteady_2nd_order_bounded
from .unsteady_global_time import unsteady_global_time
from .vof_numerics import vof_numerics
from .warped_face_gradient_correction import warped_face_gradient_correction
from .pseudo_time_method import pseudo_time_method
from .set_solution_methods_to_default import set_solution_methods_to_default
class methods(Group):
    """
    'methods' child.
    """

    fluent_name = "methods"

    child_names = \
        ['accelerated_non_iterative_time_marching',
         'convergence_acceleration_for_stretched_meshes',
         'discretization_scheme', 'expert', 'flux_type', 'frozen_flux',
         'gradient_scheme', 'high_order_term_relaxation',
         'multiphase_numerics', 'nb_gradient_boundary_option',
         'nita_expert_controls', 'noniterative_time_advance', 'overset',
         'p_v_coupling', 'phase_based_vof_discretization',
         'reduced_rank_extrapolation', 'reduced_rank_extrapolation_options',
         'residual_smoothing', 'unsteady_1st_order', 'unsteady_2nd_order',
         'unsteady_2nd_order_bounded', 'unsteady_global_time', 'vof_numerics',
         'warped_face_gradient_correction', 'pseudo_time_method']

    accelerated_non_iterative_time_marching: accelerated_non_iterative_time_marching = accelerated_non_iterative_time_marching
    """
    accelerated_non_iterative_time_marching child of methods.
    """
    convergence_acceleration_for_stretched_meshes: convergence_acceleration_for_stretched_meshes = convergence_acceleration_for_stretched_meshes
    """
    convergence_acceleration_for_stretched_meshes child of methods.
    """
    discretization_scheme: discretization_scheme = discretization_scheme
    """
    discretization_scheme child of methods.
    """
    expert: expert = expert
    """
    expert child of methods.
    """
    flux_type: flux_type = flux_type
    """
    flux_type child of methods.
    """
    frozen_flux: frozen_flux = frozen_flux
    """
    frozen_flux child of methods.
    """
    gradient_scheme: gradient_scheme = gradient_scheme
    """
    gradient_scheme child of methods.
    """
    high_order_term_relaxation: high_order_term_relaxation = high_order_term_relaxation
    """
    high_order_term_relaxation child of methods.
    """
    multiphase_numerics: multiphase_numerics = multiphase_numerics
    """
    multiphase_numerics child of methods.
    """
    nb_gradient_boundary_option: nb_gradient_boundary_option = nb_gradient_boundary_option
    """
    nb_gradient_boundary_option child of methods.
    """
    nita_expert_controls: nita_expert_controls = nita_expert_controls
    """
    nita_expert_controls child of methods.
    """
    noniterative_time_advance: noniterative_time_advance = noniterative_time_advance
    """
    noniterative_time_advance child of methods.
    """
    overset: overset = overset
    """
    overset child of methods.
    """
    p_v_coupling: p_v_coupling = p_v_coupling
    """
    p_v_coupling child of methods.
    """
    phase_based_vof_discretization: phase_based_vof_discretization = phase_based_vof_discretization
    """
    phase_based_vof_discretization child of methods.
    """
    reduced_rank_extrapolation: reduced_rank_extrapolation = reduced_rank_extrapolation
    """
    reduced_rank_extrapolation child of methods.
    """
    reduced_rank_extrapolation_options: reduced_rank_extrapolation_options = reduced_rank_extrapolation_options
    """
    reduced_rank_extrapolation_options child of methods.
    """
    residual_smoothing: residual_smoothing = residual_smoothing
    """
    residual_smoothing child of methods.
    """
    unsteady_1st_order: unsteady_1st_order = unsteady_1st_order
    """
    unsteady_1st_order child of methods.
    """
    unsteady_2nd_order: unsteady_2nd_order = unsteady_2nd_order
    """
    unsteady_2nd_order child of methods.
    """
    unsteady_2nd_order_bounded: unsteady_2nd_order_bounded = unsteady_2nd_order_bounded
    """
    unsteady_2nd_order_bounded child of methods.
    """
    unsteady_global_time: unsteady_global_time = unsteady_global_time
    """
    unsteady_global_time child of methods.
    """
    vof_numerics: vof_numerics = vof_numerics
    """
    vof_numerics child of methods.
    """
    warped_face_gradient_correction: warped_face_gradient_correction = warped_face_gradient_correction
    """
    warped_face_gradient_correction child of methods.
    """
    pseudo_time_method: pseudo_time_method = pseudo_time_method
    """
    pseudo_time_method child of methods.
    """
    command_names = \
        ['set_solution_methods_to_default']

    set_solution_methods_to_default: set_solution_methods_to_default = set_solution_methods_to_default
    """
    set_solution_methods_to_default command of methods.
    """
